/*******************************************************************************
 *  Copyright 2007 SWTBot, http://swtbot.org/
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 ******************************************************************************/
package net.sf.swtbot.eclipse.finder.finder;

import java.util.ArrayList;
import java.util.List;

import net.sf.swtbot.eclipse.finder.widgets.SWTBotView;
import net.sf.swtbot.eclipse.finder.widgets.SWTBotViewMenu;
import net.sf.swtbot.finder.UIThreadRunnable;
import net.sf.swtbot.finder.UIThreadRunnable.ListResult;
import net.sf.swtbot.matcher.IMatcher;
import net.sf.swtbot.widgets.WidgetNotFoundException;

import org.apache.commons.collections.set.ListOrderedSet;
import org.apache.log4j.Logger;
import org.eclipse.jface.action.ActionContributionItem;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.internal.ViewPane;
import org.eclipse.ui.internal.WorkbenchPartReference;

/**
 * Finds the menu items within a view.
 *
 * @author @author Stephen Paulin &lt;paulin [at] spextreme [dot] com&gt;
 * @version $Id$
 * @since 1.2
 */
public class ViewMenuFinder {
	/**
	 * The logging instance for this class.
	 */
	private static final Logger	log	= Logger.getLogger(ViewMenuFinder.class);

	/**
	 * Creates a MenuFinder.
	 */
	public ViewMenuFinder() {
		// Do nothing.
	}

	/**
	 * Finds a menu matching the given item in all available shells. If recursive is set, it will attempt to find the
	 * controls recursively in each of the menus it that is found.
	 *
	 * @param matcher the matcher that can match menus and menu items.
	 * @return all menus in all shells that match the matcher.
	 */
	public List findMenus(IMatcher matcher) {
		return findMenus(getViews(), matcher, true);
	}

	/**
	 * Finds all the menus using the given matcher in the set of shells provided. If recursive is set, it will attempt
	 * to find the controls recursively in each of the menus it that is found.
	 *
	 * @param view the views to probe for their menus.
	 * @param matcher the matcher that can match menus and menu items.
	 * @param recursive if set to true, will find sub-menus as well.
	 * @return all menus in the specified shells that match the matcher.
	 */
	public List findMenus(IViewReference[] view, IMatcher matcher, boolean recursive) {
		ListOrderedSet result = new ListOrderedSet();
		for (int i = 0; i < view.length; i++)
			result.addAll(findMenus(view[i], matcher, recursive));

		return new ArrayList(result);
	}

	/**
	 * Gets a list of all menus within the view.
	 *
	 * @param view the view to probe for menus.
	 * @param matcher the matcher that can match menus and menu items.
	 * @param recursive if set to <code>true</code>, will find sub-menus as well.
	 * @return The list of menus (IContributionItems) that match the matcher.
	 */
	public List findMenus(final IViewReference view, final IMatcher matcher, final boolean recursive) {
		return UIThreadRunnable.syncExec(new ListResult() {

			public List run() {
				ViewPane viewPane = (ViewPane) ((WorkbenchPartReference) view).getPane();
				MenuManager mgr = viewPane.getMenuManager();
				List l = new ArrayList();

				l.addAll(getMenuItemsInternal(mgr.getItems(), matcher, recursive));

				return l;
			}
		});
	}

	/**
	 * Gets all of the shells in the current display.
	 *
	 * @return all shells in the display.
	 * @see Display#getShells()
	 */
	protected IViewReference[] getViews() {
		return (IViewReference[]) SWTBotView.findViews().toArray();
	}

	// This is expected to be called from within the UI thread. If not it will throw
	// exceptions based on invalid thread access.
	private List getMenuItemsInternal(IContributionItem[] items, IMatcher matcher, boolean recursive) {
		List l = new ArrayList();

		for (int i = 0; i < items.length; i++) {
			IContributionItem item = items[i];

			try {
				if ((item instanceof MenuManager) && recursive) {
					// Sub menus
					MenuManager menuManager = (MenuManager) item;

					l.addAll(getMenuItemsInternal(menuManager.getItems(), matcher, recursive));
				} else if (item instanceof ActionContributionItem) {
					// Menus
					ActionContributionItem actionContribution = (ActionContributionItem) item;

					if (matcher.match(actionContribution.getAction()))
						l.add(new SWTBotViewMenu(actionContribution));
				}
			} catch (WidgetNotFoundException e) {
				log.warn(e);
			}
		}

		return l;
	}
}
